# -*- coding: utf-8 -*-

from django.http import Http404
from django.shortcuts import render, get_object_or_404, redirect
from django.template.loader import render_to_string

from lib.access_auth import access_restricted, none_if_anonymous, permission_needed
from portailuser.app_logger import AppLogger
from lib.tools import confirmation

from datetime import datetime

from formations.forms import FormationForm
from formations.models import Formation, prochaine_formation


# Logs
_logger = AppLogger("gestion_formations")


@access_restricted
def index(request):
    formations = []
    formations = Formation.objects.filter(ecole=request.ecole, date_heure__gt=datetime.today()).order_by("date_heure")

    c = {
        "formations": formations,
    }
    return render(request, "formations/index.html", c)


@access_restricted
def inscription(request, form_id):
    formation = get_object_or_404(Formation, pk=form_id)
    if formation.date_heure < datetime.today():
        raise Http404

    if request.method == "POST":
        if "annuler" in request.POST:
            return redirect("formations:index")

        a_portable = False

        if formation.places_fixes() > 0 and formation.places_portables() > 0:
            if "amene_portable" in request.POST:
                a_portable = True

        elif formation.places_portables() > 0:
            a_portable = True

        elif formation.places_fixes() <= 0:
            raise Http404

        formation.inscrire(request.user, a_portable)

        return redirect("formations:index")

    c = {
        "nom": formation.nom,
        "places_fixes": formation.places_fixes(),
        "places_portables": formation.places_portables(),
    }

    return render(request, "formations/inscription.html", c)


@access_restricted
def desinscription(request, form_id):
    formation = get_object_or_404(Formation, pk=form_id)

    if request.method == "POST":
        if "annuler" in request.POST:
            return redirect("formations:index")

        formation.desinscrire(request.user)
        return redirect("formations:index")

    c = {
        "nom": formation.nom,
    }

    return render(request, "formations/desinscription.html", c)


@none_if_anonymous
def widget(request):
    f = prochaine_formation(request.user)
    if not f:
        return None

    c = {
        "formation": f,
    }

    return (render_to_string("formations/widget.html", c), "bleu")


@permission_needed("gerer_formations")
def liste(request):
    formations = Formation.objects.filter(ecole=request.ecole)
    c = {
        "formations": formations.filter(date_heure__gt=datetime.today()).order_by("date_heure"),
        "formations_passees": formations.filter(date_heure__lte=datetime.today()).order_by("-date_heure"),
    }

    return render(request, "formations/liste.html", c)


@permission_needed("gerer_formations")
def edition(request, id=None):
    if id is not None:
        formation = get_object_or_404(Formation, ecole=request.ecole, id=id)
    else:
        formation = None

    if request.method == "POST":
        form = FormationForm(request.POST, instance=formation)

        if form.is_valid():
            formation = form.save(commit=False)
            formation.ecole = request.ecole
            if not formation.responsable:
                formation.responsable = request.user
            formation.save()
            _logger.log(
                request.user,
                "A édité la formation « {} » (école {})",
                formation.nom,
                formation.ecole,
            )
            return redirect("gestion:formations:liste")

    else:
        form = FormationForm(instance=formation)

    c = {
        "formation": formation,
        "form": form,
    }

    return render(request, "formations/edition.html", c)


@permission_needed("gerer_formations")
def suppression(request, id):
    formation = get_object_or_404(Formation, ecole=request.ecole, id=id)

    if request.method == "POST":
        if "conf_ok" in request.POST:
            _logger.log(
                request.user,
                "A supprimé la formation « {} » (école {})",
                formation.nom,
                formation.ecole,
            )
            formation.delete()
        return redirect("gestion:formations:liste")

    message = """Êtes-vous sûr de vouloir supprimer cette formation ?"""
    boutons = [("conf_annul", "Annuler"), ("conf_ok", "Supprimer")]

    return confirmation(request, message, boutons, "Supprimer une formation")


@permission_needed("gerer_formations")
def liste_inscrits(request, form_id):
    formation = get_object_or_404(Formation, ecole=request.ecole, id=form_id)

    inscrits = formation.inscrits.all()

    c = {
        "formation": formation,
        "inscrits": inscrits,
    }

    return render(request, "formations/inscrits.html", c)
